@@ -696,7 +696,7 @@ int https_request(https_req_t *request, https_req_response_t *response) {
         goto exit_CTX;
     }
 
-    if (!SSL_set_tlsext_host_name(ctx->ssl, connect_host)) {
+    if (!SSL_set_tlsext_host_name(ctx->ssl, request->host)) {
         netdata_log_error("Error setting TLS SNI host");
         goto exit_CTX;
     }
